@echo off

REM Copyright 2012 Leon van Dommelen.  This software is made available
REM under the GNU Public License version 3.  See folder copyrights.

REM Purpose: Switch to the short DOS path of the current folder.

REM Usage
goto start
:use
echo Usage: call cdshort.bat FILE [SEARCH [TEMP]]
echo        FILE: File to use to extract the short path name from.
echo              Should be a readable nonempty file in the current folder.
echo              Preferrably, FILE should be a string not likely to be found
echo              at random in a DOS path.
echo        SEARCH: Asku-format search string that can be found inside FILE
echo                and which will be used to verify its identity.
echo        TEMP: Folder to put a temporary batch file @cdtmp_.bat in if
echo              needed.
echo              If TEMP is omitted, FILE should be a DOS 8+3 filename.  That
echo              means that its name should 1 to 8 characters long, and its
echo              extension 0 to 3.  These should be ASCII characters that are
echo              legal in DOS filenames, and in addition should not include
echo              space, point, comma, [, ], ;, =, or +.
echo Variable L2HTOP must be set suitably before using cdshort.bat.
goto err
:start

REM Recall that ~sdp and ~sf are not necessarily truncated to the correct
REM length.  (However, as far as I have seen, that only seems to happen if the
REM final file name is not a DOS 8+3 one.  If true, this would make much of
REM this batch file redundant.  Or just a check that the specified file name
REM is an 8+3 one.)  Recall also that DOS path names with multinational
REM characters may be passed on incorrectly to non-Microsoft programs like
REM asku, Firefox, ...

REM check that we have asku
if not exist "%L2HTOP%\bin\asku.exe" goto errl2h

REM note what we are doing
echo Switching to the short DOS path of the current folder...

REM check that the file exists
if not exist "%~1" goto errfil

REM check that it is a DOS 8+3 filename
if not "%~3" == "" goto end83
"%L2HTOP%\bin\asku.exe" - - "*z:%~1:*~ig/*/:[-:.,*0:*32*O*R;=+?*128:*255][-:,*0:*32*O*R;=+?*128:*255]?:/*n*(*/:[-.][-.][-.][-.][-.][-.][-.][-.][-.:]/*n*(*q2/*)*(*/.[-.]?./*j-1/*n*(*q2/*)*(*/.[-:][-:][-:][-:]/*j-1/*n*(*q2/*)*(*)*)*)*)*(*q2/*)"
if errorlevel 2 goto errfilnm
if errorlevel 1 goto errsub
:end83

REM check that we can open it and that it has the search string
"%L2HTOP%\bin\asku.exe" - - "*k/%~1*v?/*(*)*(*q5/*)*k0/*@/\*v0?/*(*q4/*)*(*)*~i?/*qe3/*c*j0/*qe1/*/%~2/*l/*n*(*q/*)*(*)*b*q2/"
if errorlevel 6 goto errsys
if errorlevel 5 goto errfilno
if errorlevel 4 goto errfild
if errorlevel 3 goto errfilo
if errorlevel 2 goto errfilm
if errorlevel 1 goto errsub

REM check that ~sdp produces the correct short DOS path to the current folder
"%L2HTOP%\bin\asku.exe" - - "*z%~sdp1%~1*X*(0*/\%~1*X/*)*k/*o/\%~1*v?/*(*)*(*q2/*)*k0/*@/\*v0?/*(*q2/*)*(*)*~i?/*qe2/*c*j0/*qe1/*/%~2/*l/*n*(*q/*)*(*)*b*q2/"
if errorlevel 3 goto errsys
if errorlevel 2 goto dobug
if errorlevel 1 goto errsub

REM see whether the path is already short
if not "%~sdp1" == "%cd%\" goto notshort
echo ** The path "%cd%" is already short. | find " "
goto end
:notshort

REM try setting the short path
cd "%~sdp1"
if errorlevel 1 goto dobug0
goto done

REM try again
:dobug0
"%L2HTOP%\bin\asku.exe" - - "*k/%~1*v?/*(*)*(*q2/*)*k0/*@/\*v0?/*(*q2/*)*(*)*~i?/*qe2/*c*j0/*qe1/*/%~2/*l/*n*(*q/*)*(*)*b*q2/"
if errorlevel 2 goto errlost
if errorlevel 1 goto errsub
:dobug
echo Wrong path found: "%~sdp1"
if "%~3" == "" goto wrncwd
dir /o/x "%~1"
if errorlevel 1 goto errsys
echo Trouble switching, trying a bit harder...
echo Will try to extract the correct path from:
echo    "%~sf1"

REM create a batch file that sets the short folder specification
dir /o/x "%~1" | "%L2HTOP%\bin\asku.exe" - "?%~3\@cdtmp_.bat" "*/*10[-*W]/*z/*l/*n*(*1*c*j0?/*)*(*)*b*k0/%~1*(0*/*1[- ]?  ?[- ]?  ?[- ]?  ?[- ]? *([!:*127][!:*127]?*)  ?*i*?*;/*10*(1*)*)*(0*/[-*10]?*10/*)*k0/*o/*v0n/*(*k0/%~1*)*(*)*z%~sf1*X*(0*/*i\*?*X/*)*k/*o/\%~1*v?/*(*)*(*(0*/*i\*?[-]?/*)*k/*o/\%~1*v?/*(*)*(*q2/*)*)*k0/*@/\*v0?/*(*q2/*)*(*)*~i?/*qe3/*c*j0/*qe1/*/%~2/*l/*n*(*.*qe3/cd *D*(0*=*H*H*H/*)*o/*D*;*q/*)*(*)*b*q2/"
if errorlevel 3 goto errsys
if errorlevel 2 goto wrncwd
if errorlevel 1 goto errsub

REM run it
call "%%~3\@cdtmp_.bat"
if errorlevel 1 goto wrncwd0
del "%~3\@cdtmp_.bat"
goto done

REM done
:done
"%L2HTOP%\bin\asku.exe" - - "*k/%~1*v?/*(*)*(*q2/*)*k0/*@/\*v0?/*(*q2/*)*(*)*~i?/*qe2/*c*j0/*qe1/*/%~2/*l/*n*(*q/*)*(*)*b*q2/"
if errorlevel 2 goto errlost
if errorlevel 1 goto errsub
echo Path now: "%cd%" | find " "
goto end

REM warnings
:wrncwd0
del "%~3\@cdtmp_.bat"
"%L2HTOP%\bin\asku.exe" - - "*k/%~1*v?/*(*)*(*q2/*)*k0/*@/\*v0?/*(*q2/*)*(*)*~i?/*qe2/*c*j0/*qe1/*/%~2/*l/*n*(*q/*)*(*)*b*q2/"
if errorlevel 2 goto errlost
if errorlevel 1 goto errsub
:wrncwd
echo ** Unable to switch to the short DOS path. | find " "
goto end

REM errors
:errl2h
echo *** Fatal error: cannot find asku.exe in the l2h bin folder.
echo     Name L2HTOP may not be correctly set or the file is missing.
goto err
:errfil
echo *** File "%~1" not found.
goto err
:errfilnm
echo *** Error: "%~1" is not a legal DOS 8+3 filename.
echo     The name must be 1 to 8 characters long and the extension 0 to 3.
echo     They may not contain point, space, comma, [, ], ;, =, +, or any
echo     multinational (8 bit) characters.
goto err
:errfilno
echo *** File "%~1" could not be found.
echo     This might be due to multinational characters in the name.
goto err
:errfild
echo *** Error: "%~1" is a folder, not a file.
goto err
:errfilo
echo *** File "%~1" cannot be opened.  Check file protection and disk.
goto err
:errfilm
echo *** Program error: file "%~1" has no string matching "%~2".
goto err
:errlost
echo *** System error: lost location while switching to the short DOS path.
goto end
:errsub
echo *** An unexpected error occurred while processing.
echo     Please see above for clues about the problem.  Or check disk
echo     space, disk health, and file access.
goto err
:errenv
echo *** DOS environment full??  That is not supposed to happen nowadays.
goto err
:errsys
echo *** Operating system error.  Check disk space and file protections.
echo     If not, try rerunning install_l2h in the l2h_folder or contact the
echo     l2h author.
:err
echo exit | find "error"

:end
